/**
 * 
 */
package org.apache.geronimo.samples.bank.ejb;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.geronimo.samples.bank.dto.AccountDTO;
import org.apache.geronimo.samples.bank.dto.ExchangeRateDTO;

/**
 * @ejb.bean name="BankManagerFacadeBean"
 *       type="Stateless"
 *       jndi-name="org.apache.geronimo.samples.bank.ejb.BankManagerFacadeBean"
 *       local-jndi-name="java:comp/env/ejb/BankManagerFacade"
 *       view-type="both"
 *
 * @ejb.permission unchecked="true"
 *
 * @ejb.interface generate="local,remote" local-class="org.apache.geronimo.samples.bank.ejb.BankManagerFacadeLocal" remote-class="org.apache.geronimo.samples.bank.ejb.BankManagerFacade"
 * @ejb.home generate="local,remote" local-class="org.apache.geronimo.samples.bank.ejb.BankManagerFacadeHomeLocal" remote-class="org.apache.geronimo.samples.bank.ejb.BankManagerFacadeHome"
 * @ejb.util generate="physical"
 * 
 * @ejb.ejb-ref ejb-name="Customer" view-type="local" ref-name="ejb/Customer"
 * @ejb.ejb-ref ejb-name="Account" view-type="local" ref-name="ejb/Account"
 * @ejb.ejb-ref ejb-name="ExchangeRate" view-type="local" ref-name="ejb/ExchangeRate"
 * 
 * @web.ejb-local-ref
 *    name="ejb/BankManagerFacade"
 *    type="Session"
 *    home="org.apache.geronimo.samples.bank.ejb.BankManagerFacadeHomeLocal"
 *    local="org.apache.geronimo.samples.bank.ejb.BankManagerFacadeLocal"
 *    link="BankManagerFacadeBean"
 * 
 * @author Lasantha Ranaweera
 */

public class BankManagerFacadeBean implements SessionBean {
	
	/**
	 * 
	 * @ejb.interface-method view-type="local"
	 * @return
	 */
	public Collection getAccountInformation(String customerId){
		Context context = null;
		ArrayList accountBeanList = new ArrayList();
		try {
			context = new InitialContext();
			CustomerHomeLocal customerHome = (CustomerHomeLocal)context.lookup(CustomerHomeLocal.JNDI_NAME);
			CustomerLocal customer = customerHome.findByPrimaryKey(customerId);
			Iterator accountIterator = customer.getAccounts().iterator();	
			
			
			while(accountIterator.hasNext()){
				AccountDTO account = new AccountDTO();
				
				AccountLocal accountLocal = (AccountLocal)accountIterator.next();
				account.setAccountNumber(accountLocal.getAccountNumber());
				account.setBalance(accountLocal.getBalance());
				account.setAccountType(accountLocal.getAccountType());
				
				accountBeanList.add(account);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				context.close();
			} catch (NamingException e) {
				e.printStackTrace();
			}
		}
		
		return accountBeanList;
	}
	
	/**
	 * @ejb.interface-method view-type="local"
	 * @return
	 */	
	public Collection getExchangeRates() {
		Context context = null;
		Collection rateList = new ArrayList();
		try {
			context = new InitialContext();
			ExchangeRateHomeLocal exchangeRateHome = (ExchangeRateHomeLocal)context.lookup(ExchangeRateHomeLocal.JNDI_NAME);
			Collection rates = exchangeRateHome.findByAll();
			
			Iterator ratesIterator = rates.iterator();
			
			while(ratesIterator.hasNext()){
				ExchangeRateLocal exRate = (ExchangeRateLocal)ratesIterator.next();
				ExchangeRateDTO rateDTO = new ExchangeRateDTO();
				
				rateDTO.setRateId(exRate.getRateId());
				rateDTO.setCurrency(exRate.getCurrency());
				rateDTO.setRate(exRate.getRate());
				
				rateList.add(rateDTO);
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				context.close();
			} catch (NamingException e) {
				e.printStackTrace();
			}
		}
		return rateList;
	}

	
	/**
	 * @ejb.interface-method view-type="local"
	 * @param customerCode
	 * @return
	 */
	public CustomerLocal getCustomer(String customerCode){
		Context context = null;
		CustomerLocal customer = null;
		
		try {
			context = new InitialContext();
			CustomerHomeLocal customerHome = (CustomerHomeLocal)context.lookup(CustomerHomeLocal.JNDI_NAME);
			customer  = customerHome.findByPrimaryKey(customerCode);
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				context.close();
			} catch (NamingException e) {
				e.printStackTrace();
			}
		}
		return customer;
	}

	
	/**
	 * @ejb.interface-method view-type="remote"
	 * @param accountNo
	 * @param balance
	 */
	public void changeAccountBalance(String accountNo,Double balance){
		Context context = null;
		
		try {
			context = new InitialContext();
			AccountHomeLocal accountHome = (AccountHomeLocal)context.lookup(AccountHomeLocal.JNDI_NAME);
			AccountLocal account = accountHome.findByPrimaryKey(accountNo);
			account.setBalance(balance);			
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				context.close();
			} catch (NamingException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	/**
	 * @ejb.interface-method view-type="remote"
	 * @param accountNo
	 */
	public Double getAccountBalance(String accountNo){
		Context context = null;
		Double balance = null;
		try {
			context = new InitialContext();
			AccountHomeLocal accountHome = (AccountHomeLocal)context.lookup(AccountHomeLocal.JNDI_NAME);
			AccountLocal account = accountHome.findByPrimaryKey(accountNo);
			balance = account.getBalance();			
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				context.close();
			} catch (NamingException e) {
				e.printStackTrace();
			}
		}
		return balance;
	}
	
	/**
	 * @ejb.create-method
	 * 
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}

	public void ejbPostCreate() throws javax.ejb.CreateException {
	}

	protected javax.ejb.SessionContext context = null;

	public void setSessionContext(javax.ejb.SessionContext ctx) {
		this.context = ctx;
	}

	protected javax.ejb.SessionContext getSessionContext() {
		return this.context;		
	}

	public void ejbActivate() throws EJBException, RemoteException {

	}

	public void ejbPassivate() throws EJBException, RemoteException {

	}

	public void ejbRemove() throws EJBException, RemoteException {

	}

}
