package org.apache.geronimo.samples.bank.ejb;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;

import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.NoSuchEntityException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * 
 * @ejb.bean type="BMP" bmp-version="2.x" name="ExchangeRate"
 *           local-jndi-name="java:comp/env/ejb/ExchangeRate" view-type="local"
 *           primkey-field="rateId"
 * 
 * @ejb.interface generate="local"
 *                local-class="org.apache.geronimo.samples.bank.ejb.ExchangeRateLocal"
 * @ejb.home generate="local"
 *           local-class="org.apache.geronimo.samples.bank.ejb.ExchangeRateHomeLocal"
 *           
 * @ejb.transaction type="Required"
 * 
 * @web.ejb-local-ref name="ejb/ExchangeRate" type="Entity"
 *                    home="org.apache.geronimo.samples.bank.ejb.ExchangeRateHomeLocal"
 *                    local="org.apache.geronimo.samples.bank.ejb.ExchangeRateLocal"
 *                    link="ExchangeRate"
 *                    
 * @ejb.resource-ref description="DataSource for BankDB" res-ref-name="jdbc/BankDataSource" 
 * 					 res-type="javax.sql.DataSource" res-auth="Container" 
 * 
 * @ejb.persistence table-name="exchange_rate"
 * 
 * @author Lasantha Ranaweera
 * 
 */
public abstract class ExchangeRateBean implements EntityBean {

	private Connection connection = null;

	private String rateId;

	private String currency;

	private Double rate;

	private EntityContext context;

	/**
	 * @ejb.interface-method view-type="local" 
	 * @return
	 */
	public String getCurrency() {
		return currency;
	}

	/**
	 * @ejb.interface-method view-type="local"
	 * @param currency
	 */
	public void setCurrency(String currency) {
		this.currency = currency;
	}

	/**
	 * @ejb.interface-method view-type="local" 
	 * @return
	 */
	public Double getRate() {
		return rate;
	}

	/**
	 * @ejb.interface-method view-type="local"
	 * @param rate
	 */
	public void setRate(Double rate) {
		this.rate = rate;
	}

	/**
	 * @ejb.pk-field
	 * @ejb.interface-method view-type="local"
	 * @return
	 */
	public String getRateId() {
		return rateId;
	}

	/**
	 * @ejb.interface-method view-type="local"
	 * @param rateId
	 */
	public void setRateId(String rateId) {
		this.rateId = rateId;
	}

	/**
	 * @ejb.create-method
	 */
	public String ejbCreate(String rateId, String currency, Double rate)
			throws javax.ejb.CreateException {
		try {
			this.rateId = rateId;
			this.currency = currency;
			this.rate = rate;

			openConnection();
			insertRow();
			closeConnection();
		} catch (Exception e) {
			throw new EJBException("ejbCreate: " + e.getMessage());
		}
		return rateId;
	}

	public void ejbPostCreate(String rateId, String currency, Double rate)
			throws javax.ejb.CreateException {
	}

	public String ejbFindByPrimaryKey(String primaryKey) throws FinderException {
		boolean status = false;

		try {
			openConnection();
			status = selectByPrimaryKey(primaryKey);
			closeConnection();

		} catch (Exception e) {
			throw new EJBException("ejbFindByPrimaryKey: " + e.getMessage());
		}

		if (status) {
			return primaryKey;
		} else {
			throw new ObjectNotFoundException();
		}
	}

	public Collection ejbFindByAll() throws FinderException {
		Collection result = null;
		try {
			openConnection();
			result = selectAll();
			closeConnection();
		} catch (Exception e) {
			throw new EJBException("ejbFindByAll: " + e.getMessage());
		}

		if (result.isEmpty()) {
			throw new ObjectNotFoundException("No rows found.");
		} else {
			return result;
		}
	}

	public void ejbActivate() throws EJBException, RemoteException {
		rateId = (String)context.getPrimaryKey();
	}

	public void ejbLoad() throws EJBException, RemoteException {
		try {
			openConnection();
			loadRow();
			closeConnection();
		} catch (Exception e) {
			throw new EJBException("ejbLoad: " + e.getMessage());
		}

	}

	public void ejbPassivate() throws EJBException, RemoteException {

	}

	public void ejbRemove() throws RemoveException, EJBException,
			RemoteException {
		try {
			openConnection();
			deleteRow();
			closeConnection();
		} catch (Exception e) {
			throw new EJBException("ejbRemove: " + e.getMessage());
		}

	}

	public void ejbStore() throws EJBException, RemoteException {
		try {
			openConnection();
			storeRow();
			closeConnection();
		} catch (Exception e) {
			throw new EJBException("ejbStore: " + e.getMessage());
		}
	}

	public void setEntityContext(EntityContext ctx) throws EJBException,
			RemoteException {
		this.context = ctx;
	}

	public void unsetEntityContext() throws EJBException, RemoteException {
		this.context = null;
	}

	private void openConnection() throws NamingException, SQLException {
		Context context = new InitialContext();
		DataSource ds = (DataSource) context.lookup("java:comp/env/jdbc/BankDataSource");
		connection = ds.getConnection();
	}

	private void closeConnection() throws SQLException {
		connection.close();
	}

	private void insertRow() throws SQLException {
		String SQL = "INSERT INTO exchange_rate (RATE_ID,CURRENCY,RATE) VALUES (?,?,?)";
		PreparedStatement pStmt = connection.prepareStatement(SQL);
		pStmt.setString(1, rateId);
		pStmt.setString(2, currency);
		pStmt.setDouble(3, rate.doubleValue());

		pStmt.executeUpdate();
		pStmt.close();
	}

	private void deleteRow() throws SQLException {
		String SQL = "DELETE FROM exchange_rate WHERE RATE_ID = ?";
		PreparedStatement pStmt = connection.prepareStatement(SQL);
		pStmt.setString(1, rateId);
		pStmt.executeUpdate();
		pStmt.close();
	}

	private boolean selectByPrimaryKey(String key) throws SQLException {
		String SQL = "SELECT RATE_ID,CURRENCY,RATE FROM exchange_rate WHERE RATE_ID = ?";
		PreparedStatement pStmt = connection.prepareStatement(SQL);
		pStmt.setString(1, key);

		ResultSet rs = pStmt.executeQuery();

		boolean status = rs.next();
		rs.close();
		pStmt.close();

		return status;
	}

	private Collection selectAll() throws SQLException {
		String SQL = "SELECT RATE_ID,CURRENCY,RATE FROM exchange_rate";

		ArrayList dataSet = new ArrayList();

		PreparedStatement pStmt = connection.prepareStatement(SQL);

		ResultSet rs = pStmt.executeQuery();

		while (rs.next()) {
			dataSet.add(rs.getString(1));
		}

		rs.close();
		pStmt.close();

		return dataSet;
	}

	private void loadRow() throws SQLException {
		String SQL = "SELECT RATE_ID,CURRENCY,RATE FROM exchange_rate WHERE RATE_ID = ?";
		PreparedStatement pStmt = connection.prepareStatement(SQL);
		pStmt.setString(1, rateId);

		ResultSet rs = pStmt.executeQuery();
		
		if(rs.next()){
			this.rateId = rs.getString(1);
			this.currency = rs.getString(2);
			this.rate = new Double(rs.getDouble(3));
		}

		rs.close();
		pStmt.close();

	}
	
	private void storeRow() throws SQLException {
		String SQL = "UPDATE exchange_rate SET RATE_ID = ?, CURRENCY = ?, RATE = ? WHERE RATE_ID = ?";
		int status = 0;
		PreparedStatement pStmt = connection.prepareStatement(SQL);
		pStmt.setString(1, rateId);
		pStmt.setString(2,currency);
		pStmt.setDouble(3, rate.doubleValue());
		pStmt.setString(4, rateId);
		
		status = pStmt.executeUpdate();
		if(status == 0){
			throw new NoSuchEntityException("Storing row for id "+rateId+" failed ");
		}
		
		pStmt.close();
	}

}
